#!/usr/bin/env bash

# Basic system information
IS_DARWIN=$(uname -s | grep -i -c darwin)
IS_LINUX=$(uname -s | grep -i -c linux)
IS_X86_64=$(uname -m | grep -i -c x86_64)

# Base options to test. This list is taken from the Configure script.
#  Removed options include "asm" and "unit-test"
OPTIONS=("aes" "afalgeng" "async" "autoalginit" "autoerrinit" "bf"
	"blake2" "camellia" "capieng" "cast" "chacha" "cmac" "cms" "comp"
	"crypto-mdebug" "crypto-mdebug-backtrace" "ct" "deprecated" "des" "dgram"
	"dh" "dsa" "dso" "dtls" "dynamic-engine" "ec" "ec2m" "ecdh" "ecdsa"
	"ec_nistp_64_gcc_128" "egd" "engine" "err" "filenames" "gost"
	"heartbeats" "hmac" "hw" "idea" "md2" "md4" "md5" "mdc2"
	"multiblock" "nextprotoneg" "ocb" "ocsp" "pic" "poly1305" "posix-io"
	"psk" "rc2" "rc4" "rc5" "rdrand" "rfc3779" "ripemd" "rmd160" "rsa"
	"scrypt" "sct" "sctp" "seed" "sha" "shared" "sock" "srp" "srtp"	"sse2"
	"ssl" "ssl-trace" "static-engine" "stdio" "threads" "tls" "ts" "ui"
	"whirlpool" "weak-ssl-ciphers" "zlib" "zlib-dynamic")
	
# ec_nistp_64_gcc_128 has some specific test conditions.
if [ "$IS_X86_64" -eq "1" ]; then
	OPTIONS+=("ec_nistp_64_gcc_128")
fi

# IPv4 and IPv6 options.
HAS_IPV4=$(ifconfig | grep -i -c '127.0.0.1')
if [ "$HAS_IPV4" -ne "0" ]; then
	OPTIONS+=("-DOPENSSL_USE_IPV4=0")
fi

HAS_IPV6=$(ifconfig | grep -i -c '::1')
if [ "$HAS_IPV6" -ne "0" ]; then
	OPTIONS+=("-DOPENSSL_USE_IPV6=0")
fi

# DEBUG and NDEBUG for Windows users
OPTIONS+=("-DDEBUG")
OPTIONS+=("-DNDEBUG")

# Add the SSL/TLS version to the mix. The Configure script cascades these.
OPTIONS+=("ssl2")
OPTIONS+=("ssl3")
OPTIONS+=("tls1")
OPTIONS+=("tls1_1")
OPTIONS+=("tls1_2")
OPTIONS+=("dtls1")
OPTIONS+=("dtls1_1")
OPTIONS+=("dtls1_2")

# CPU is logical count, memory is in MB. Low resource boards have
#   fewer than 4 cores and 1GB or less memory. We use this to
#   determine if we can build in parallel without an OOM kill.
CPU_COUNT=0
MEM_SIZE=0
if [ "$IS_LINUX" -ne "0" ] && [ -e "/proc/cpuinfo" ]; then
	CPU_COUNT=$(cat /proc/cpuinfo | grep -c '^processor')
	MEM_SIZE=$(cat /proc/meminfo | grep "MemTotal" | awk '{print $2}')
	MEM_SIZE=$(($MEM_SIZE/1024))
elif [ "$IS_DARWIN" -ne "0" ]; then
	CPU_COUNT=$(sysctl -a 2>/dev/null | grep 'hw.availcpu' | head -1 | awk '{print $3}')
	MEM_SIZE=$(sysctl -a 2>/dev/null | grep 'hw.memsize' | head -1 | awk '{print $3}')
	MEM_SIZE=$(($MEM_SIZE/1024/1024))
fi


# Some ARM devboards cannot use 'make -j N', even with multiple cores and RAM
#  An 8-core Cubietruck Plus with 2GB RAM experiences OOM kills with '-j 2'.
HAVE_SWAP=1
if [ "$IS_LINUX" -ne "0" ]; then
	if [ -e "/proc/meminfo" ]; then
		SWAP_SIZE=$(cat /proc/meminfo | grep "SwapTotal" | awk '{print $2}')
		if [ "$SWAP_SIZE" -eq "0" ]; then
			HAVE_SWAP=0
		fi
	else
		HAVE_SWAP=0
	fi
fi


# Determine MAKE options
#   If testing on the BSDs, you may need to change this to gmake.
MAKE=make
if [ "$CPU_COUNT" -ge "2" ] && [ "$MEM_SIZE" -ge "1280" ] && [ "$HAVE_SWAP" -ne "0" ]; then
	MAKEARGS=(-j "$CPU_COUNT")
	echo "Using $MAKE -j $CPU_COUNT"
fi


# Avoid the 32/64-bit timeout on OS X
if [ "$IS_DARWIN" -ne "0" ]; then
	if [ "$IS_X86_64" -eq "1" ]; then
		export KERNEL_BITS=64
	else
		export KERNEL_BITS=32
	fi
fi


# Header with all options in the list
echo "****************************************"
echo "Testing options: " ${OPTIONS[@]}
echo

# A one-off
echo "****************************************"
echo "Testing no options"

./config
"$MAKE" clean >/dev/null 2>&1
"$MAKE" depend >/dev/null 2>&1

"$MAKE" "${MAKEARGS[@]}"
"$MAKE" test "${MAKEARGS[@]}"

if [ "$?" -ne "0" ]; then
	echo "  !!!!!!!!!!FAILED!!!!!!!!!!"
fi

echo

# A one-off
echo "****************************************"
echo "Testing no-asm"

./config no-asm
"$MAKE" clean >/dev/null 2>&1
"$MAKE" depend >/dev/null 2>&1

"$MAKE" "${MAKEARGS[@]}"
"$MAKE" test "${MAKEARGS[@]}"

if [ "$?" -ne "0" ]; then
	echo "  !!!!!!!!!!FAILED!!!!!!!!!!"
fi

echo

# A one-off
echo "****************************************"
echo "Testing no-ssl2 no-ssl3 no-comp"

./config no-ssl2 no-ssl3 no-comp
"$MAKE" clean >/dev/null 2>&1
"$MAKE" depend >/dev/null 2>&1

"$MAKE" "${MAKEARGS[@]}"
"$MAKE" test "${MAKEARGS[@]}"

if [ "$?" -ne "0" ]; then
	echo "  !!!!!!!!!!FAILED!!!!!!!!!!"
fi

echo

for opt in ${OPTIONS[@]}; do

	echo "****************************************"
	echo "Testing \"no-$opt\""

	./config no-$opt
	"$MAKE" clean >/dev/null 2>&1
	"$MAKE" depend >/dev/null 2>&1

	"$MAKE" "${MAKEARGS[@]}"
	"$MAKE" test "${MAKEARGS[@]}"
	
	if [ "$?" -ne "0" ]; then
		echo "  !!!!!!!!!!FAILED!!!!!!!!!!"
	fi

	echo
	
	echo "****************************************"
	echo "Testing \"no-asm no-$opt\""

	./config no-asm no-$opt
	"$MAKE" clean >/dev/null 2>&1
	"$MAKE" depend >/dev/null 2>&1

	"$MAKE" "${MAKEARGS[@]}"
	"$MAKE" test "${MAKEARGS[@]}"
	
	if [ "$?" -ne "0" ]; then
		echo "  !!!!!!!!!!FAILED!!!!!!!!!!"
	fi

	echo
done
